<?php

namespace Mmc\Music\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Mmc\Music\Http\Requests\AdminRequest;
use Mmc\Music\Interfaces\AdminRepositoryInterface;
use Mmc\Music\Models\Admin;

/**
 * Resource controller class for admin.
 */
class AdminResourceController extends BaseController
{

    /**
     * Initialize admin resource controller.
     *
     * @param type AdminRepositoryInterface $admin
     *
     * @return null
     */
    public function __construct(AdminRepositoryInterface $admin)
    {
        parent::__construct();
        $this->repository = $admin;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Mmc\Music\Repositories\Criteria\AdminResourceCriteria::class);
    }

    /**
     * Display a list of admin.
     *
     * @return Response
     */
    public function index(AdminRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Mmc\Music\Repositories\Presenter\AdminPresenter::class)
                ->$function();
        }

        $admins = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('music::admin.names'))
            ->view('music::admin.index', true)
            ->data(compact('admins', 'view'))
            ->output();
    }

    /**
     * Display admin.
     *
     * @param Request $request
     * @param Model   $admin
     *
     * @return Response
     */
    public function show(AdminRequest $request, Admin $admin)
    {

        if ($admin->exists) {
            $view = 'music::admin.show';
        } else {
            $view = 'music::admin.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('music::admin.name'))
            ->data(compact('admin'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new admin.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AdminRequest $request)
    {

        $admin = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('music::admin.name')) 
            ->view('music::admin.create', true) 
            ->data(compact('admin'))
            ->output();
    }

    /**
     * Create new admin.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdminRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $admin                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('music::admin.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/admin/' . $admin->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/music/admin'))
                ->redirect();
        }

    }

    /**
     * Show admin for editing.
     *
     * @param Request $request
     * @param Model   $admin
     *
     * @return Response
     */
    public function edit(AdminRequest $request, Admin $admin)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('music::admin.name'))
            ->view('music::admin.edit', true)
            ->data(compact('admin'))
            ->output();
    }

    /**
     * Update the admin.
     *
     * @param Request $request
     * @param Model   $admin
     *
     * @return Response
     */
    public function update(AdminRequest $request, Admin $admin)
    {
        try {
            $attributes = $request->all();

            $admin->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('music::admin.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/admin/' . $admin->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/admin/' . $admin->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the admin.
     *
     * @param Model   $admin
     *
     * @return Response
     */
    public function destroy(AdminRequest $request, Admin $admin)
    {
        try {

            $admin->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::admin.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('music/admin/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/admin/' . $admin->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple admin.
     *
     * @param Model   $admin
     *
     * @return Response
     */
    public function delete(AdminRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::admin.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('music/admin'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/admin'))
                ->redirect();
        }

    }

    /**
     * Restore deleted admins.
     *
     * @param Model   $admin
     *
     * @return Response
     */
    public function restore(AdminRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('music::admin.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/music/admin'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/admin/'))
                ->redirect();
        }

    }

}
