<?php

namespace Mmc\Music\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Mmc\Music\Http\Requests\FileRequest;
use Mmc\Music\Interfaces\FileRepositoryInterface;
use Mmc\Music\Models\File;

/**
 * Resource controller class for file.
 */
class FileResourceController extends BaseController
{

    /**
     * Initialize file resource controller.
     *
     * @param type FileRepositoryInterface $file
     *
     * @return null
     */
    public function __construct(FileRepositoryInterface $file)
    {
        parent::__construct();
        $this->repository = $file;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Mmc\Music\Repositories\Criteria\FileResourceCriteria::class);
    }

    /**
     * Display a list of file.
     *
     * @return Response
     */
    public function index(FileRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Mmc\Music\Repositories\Presenter\FilePresenter::class)
                ->$function();
        }

        $files = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('music::file.names'))
            ->view('music::file.index', true)
            ->data(compact('files', 'view'))
            ->output();
    }

    /**
     * Display file.
     *
     * @param Request $request
     * @param Model   $file
     *
     * @return Response
     */
    public function show(FileRequest $request, File $file)
    {

        if ($file->exists) {
            $view = 'music::file.show';
        } else {
            $view = 'music::file.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('music::file.name'))
            ->data(compact('file'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new file.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FileRequest $request)
    {

        $file = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('music::file.name')) 
            ->view('music::file.create', true) 
            ->data(compact('file'))
            ->output();
    }

    /**
     * Create new file.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FileRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $file                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('music::file.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/file/' . $file->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/music/file'))
                ->redirect();
        }

    }

    /**
     * Show file for editing.
     *
     * @param Request $request
     * @param Model   $file
     *
     * @return Response
     */
    public function edit(FileRequest $request, File $file)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('music::file.name'))
            ->view('music::file.edit', true)
            ->data(compact('file'))
            ->output();
    }

    /**
     * Update the file.
     *
     * @param Request $request
     * @param Model   $file
     *
     * @return Response
     */
    public function update(FileRequest $request, File $file)
    {
        try {
            $attributes = $request->all();

            $file->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('music::file.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/file/' . $file->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/file/' . $file->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the file.
     *
     * @param Model   $file
     *
     * @return Response
     */
    public function destroy(FileRequest $request, File $file)
    {
        try {

            $file->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::file.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('music/file/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/file/' . $file->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple file.
     *
     * @param Model   $file
     *
     * @return Response
     */
    public function delete(FileRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::file.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('music/file'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/file'))
                ->redirect();
        }

    }

    /**
     * Restore deleted files.
     *
     * @param Model   $file
     *
     * @return Response
     */
    public function restore(FileRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('music::file.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/music/file'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/file/'))
                ->redirect();
        }

    }

}
