<?php

namespace Mmc\Music\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Mmc\Music\Http\Requests\RegistrationRequest;
use Mmc\Music\Interfaces\RegistrationRepositoryInterface;
use Mmc\Music\Models\Registration;

/**
 * Resource controller class for registration.
 */
class RegistrationResourceController extends BaseController
{

    /**
     * Initialize registration resource controller.
     *
     * @param type RegistrationRepositoryInterface $registration
     *
     * @return null
     */
    public function __construct(RegistrationRepositoryInterface $registration)
    {
        parent::__construct();
        $this->repository = $registration;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Mmc\Music\Repositories\Criteria\RegistrationResourceCriteria::class);
    }

    /**
     * Display a list of registration.
     *
     * @return Response
     */
    public function index(RegistrationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Mmc\Music\Repositories\Presenter\RegistrationPresenter::class)
                ->$function();
        }

        $registrations = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('music::registration.names'))
            ->view('music::registration.index', true)
            ->data(compact('registrations', 'view'))
            ->output();
    }

    /**
     * Display registration.
     *
     * @param Request $request
     * @param Model   $registration
     *
     * @return Response
     */
    public function show(RegistrationRequest $request, Registration $registration)
    {

        if ($registration->exists) {
            $view = 'music::registration.show';
        } else {
            $view = 'music::registration.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('music::registration.name'))
            ->data(compact('registration'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new registration.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RegistrationRequest $request)
    {

        $registration = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('music::registration.name')) 
            ->view('music::registration.create', true) 
            ->data(compact('registration'))
            ->output();
    }

    /**
     * Create new registration.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RegistrationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $registration                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('music::registration.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/registration/' . $registration->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/music/registration'))
                ->redirect();
        }

    }

    /**
     * Show registration for editing.
     *
     * @param Request $request
     * @param Model   $registration
     *
     * @return Response
     */
    public function edit(RegistrationRequest $request, Registration $registration)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('music::registration.name'))
            ->view('music::registration.edit', true)
            ->data(compact('registration'))
            ->output();
    }

    /**
     * Update the registration.
     *
     * @param Request $request
     * @param Model   $registration
     *
     * @return Response
     */
    public function update(RegistrationRequest $request, Registration $registration)
    {
        try {
            $attributes = $request->all();

            $registration->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('music::registration.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('music/registration/' . $registration->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/registration/' . $registration->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the registration.
     *
     * @param Model   $registration
     *
     * @return Response
     */
    public function destroy(RegistrationRequest $request, Registration $registration)
    {
        try {

            $registration->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::registration.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('music/registration/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('music/registration/' . $registration->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple registration.
     *
     * @param Model   $registration
     *
     * @return Response
     */
    public function delete(RegistrationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('music::registration.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('music/registration'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/registration'))
                ->redirect();
        }

    }

    /**
     * Restore deleted registrations.
     *
     * @param Model   $registration
     *
     * @return Response
     */
    public function restore(RegistrationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('music::registration.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/music/registration'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/music/registration/'))
                ->redirect();
        }

    }

}
